@@ -137,14 +137,9 @@ class Grilo(GObject.GObject):
             with self.tracker.handler_block(self.notification_handler):
                 for media in changedMedias:
                     media_id = media.get_id()
-                    if changeType == Grl.SourceChangeType.ADDED:
-                        # Check that this media is an audio file
-                        mime_type = self.tracker.query_sync(
-                            Query.is_audio(media_id),
-                            [Grl.METADATA_KEY_MIME],
-                            self.options)[0].get_mime()
-                        if mime_type and mime_type.startswith("audio"):
-                            self.changed_media_ids.append(media_id)
+                    if (changeType == Grl.SourceChangeType.ADDED
+                            and media.is_audio()):
+                        self.changed_media_ids.append(media_id)
                     if changeType == Grl.SourceChangeType.REMOVED:
                         # There is no way to check that removed item is a media
                         # so always do the refresh
@@ -259,9 +254,29 @@ class Grilo(GObject.GObject):
 
     @log
     def populate_playlists(self, offset, callback, count=-1):
+        """Asynchronously get playlists (user and smart ones)
+
+        :param int offset: start index
+        :param function callback: callback function
+        :param int count: limit number of results
+        """
         if self.tracker:
-            GLib.idle_add(self.populate_items, Query.all_playlists(), offset,
-                                                callback, count)
+            GLib.idle_add(
+                self.populate_items, Query.all_playlists(), offset, callback,
+                count)
+
+    @log
+    def populate_user_playlists(self, offset, callback, count=-1):
+        """Asynchronously get user playlists
+
+        :param int offset: start index
+        :param function callback: callback function
+        :param int count: limit number of results
+        """
+        if self.tracker:
+            GLib.idle_add(
+                self.populate_items, Query.all_user_playlists(), offset,
+                callback, count)
 
     @log
     def populate_album_songs(self, album, callback, count=-1):
@@ -467,45 +482,5 @@ class Grilo(GObject.GObject):
         self.sparqltracker.query_async(Query.all_songs_count(), None,
                                        songs_query_cb, None)
 
-    @log
-    def playlists_available(self, callback):
-        """Checks if there are any non-static playlists available
-
-        Calls a callback function with True or False depending on the
-        availability of playlists.
-        :param callback: Function to call on result
-        """
-        def cursor_next_cb(conn, res, data):
-            try:
-                has_next = conn.next_finish(res)
-            except GLib.Error as err:
-                logger.warning("Error: {},{}".format(err.__class__, err))
-                callback(False)
-                return
-
-            if has_next:
-                count = conn.get_integer(0)
-
-                if count > 0:
-                    callback(True)
-                    return
-
-            callback(False)
-
-        def playlists_query_cb(conn, res, data):
-            try:
-                cursor = conn.query_finish(res)
-            except GLib.Error as err:
-                logger.warning("Error: {}, {}".format(err.__class__, err))
-                callback(False)
-                return
-
-            cursor.next_async(None, cursor_next_cb, None)
-
-        # TODO: currently just checks tracker, should work with any
-        # queryable supported Grilo source.
-        self.sparqltracker.query_async(
-                Query.all_non_static_playlists_count(),
-                None, playlists_query_cb, None)
 
 grilo = Grilo()
