@@ -46,7 +46,8 @@ logger = logging.getLogger(__name__)
 playlists = Playlists.get_default()
 
 
-class RepeatType:
+class RepeatMode(IntEnum):
+    """Enum for player repeat mode"""
     NONE = 0
     SONG = 1
     ALL = 2
@@ -81,7 +82,6 @@ class Player(GObject.GObject):
         'volume-changed': (GObject.SignalFlags.RUN_FIRST, None, ()),
         'prev-next-invalidated': (GObject.SignalFlags.RUN_FIRST, None, ()),
         'seeked': (GObject.SignalFlags.RUN_FIRST, None, (int,)),
-        'state-changed': (GObject.SignalFlags.RUN_FIRST, None, ()),
     }
 
     def __repr__(self):
@@ -121,7 +121,6 @@ class Player(GObject.GObject):
         self._player = GstPlayer()
         self._player.connect('clock-tick', self._on_clock_tick)
         self._player.connect('eos', self._on_eos)
-        self._player.connect('notify::state', self._on_state_change)
 
         root_window = parent_window.get_toplevel()
         self._inhibit_suspend = InhibitSuspend(root_window, self)
@@ -241,20 +240,20 @@ class Player(GObject.GObject):
 
         next_song = None
 
-        if self.repeat == RepeatType.SONG:
+        if self.repeat == RepeatMode.SONG:
             if iter_:
                 next_song = iter_
             else:
                 next_song = self.playlist.get_iter_first()
-        elif self.repeat == RepeatType.ALL:
+        elif self.repeat == RepeatMode.ALL:
             if iter_:
                 next_song = self.playlist.iter_next(iter_)
             if not next_song:
                 next_song = self.playlist.get_iter_first()
-        elif self.repeat == RepeatType.NONE:
+        elif self.repeat == RepeatMode.NONE:
             if iter_:
                 next_song = self.playlist.iter_next(iter_)
-        elif self.repeat == RepeatType.SHUFFLE:
+        elif self.repeat == RepeatMode.SHUFFLE:
             next_song = self._get_random_iter(iter_)
             if iter_:
                 self._shuffle_history.append(iter_)
@@ -287,20 +286,20 @@ class Player(GObject.GObject):
 
         previous_song = None
 
-        if self.repeat == RepeatType.SONG:
+        if self.repeat == RepeatMode.SONG:
             if iter_:
                 previous_song = iter_
             else:
                 previous_song = self.playlist.get_iter_first()
-        elif self.repeat == RepeatType.ALL:
+        elif self.repeat == RepeatMode.ALL:
             if iter_:
                 previous_song = self.playlist.iter_previous(iter_)
             if not previous_song:
                 previous_song = get_last_iter()
-        elif self.repeat == RepeatType.NONE:
+        elif self.repeat == RepeatMode.NONE:
             if iter_:
                 previous_song = self.playlist.iter_previous(iter_)
-        elif self.repeat == RepeatType.SHUFFLE:
+        elif self.repeat == RepeatMode.SHUFFLE:
             if iter_:
                 if (self._player.position < 5
                         and len(self._shuffle_history) > 0):
@@ -326,13 +325,13 @@ class Player(GObject.GObject):
 
     @log
     def has_next(self):
-        repeat_types = [RepeatType.ALL, RepeatType.SONG, RepeatType.SHUFFLE]
+        repeat_modes = [RepeatMode.ALL, RepeatMode.SONG, RepeatMode.SHUFFLE]
         if (not self.playlist
                 or self.playlist.iter_n_children(None) < 1):
             return False
         elif not self.current_song:
             return False
-        elif self.repeat in repeat_types:
+        elif self.repeat in repeat_modes:
             return True
         elif self.current_song.valid():
             tmp = self.playlist.get_iter(self.current_song.get_path())
@@ -342,13 +341,13 @@ class Player(GObject.GObject):
 
     @log
     def has_previous(self):
-        repeat_types = [RepeatType.ALL, RepeatType.SONG, RepeatType.SHUFFLE]
+        repeat_modes = [RepeatMode.ALL, RepeatMode.SONG, RepeatMode.SHUFFLE]
         if (not self.playlist
                 or self.playlist.iter_n_children(None) < 1):
             return False
         elif not self.current_song:
             return False
-        elif self.repeat in repeat_types:
+        elif self.repeat in repeat_modes:
             return True
         elif self.current_song.valid():
             tmp = self.playlist.get_iter(self.current_song.get_path())
@@ -357,7 +356,6 @@ class Player(GObject.GObject):
             return True
 
     @GObject.Property
-    @log
     def playing(self):
         """Returns if a song is currently played
 
@@ -366,25 +364,6 @@ class Player(GObject.GObject):
         """
         return self._player.state == Playback.PLAYING
 
-    @log
-    def _on_state_change(self, klass, arguments):
-        self.emit('state-changed')
-
-        return True
-
-    @log
-    def set_playing(self, value):
-        """Set state
-
-        :param bool value: Playing
-        """
-        if value:
-            self.play()
-        else:
-            self.pause()
-
-        self.emit('state-changed')
-
     @log
     def _load(self, media):
         self._time_stamp = int(time.time())
@@ -443,7 +422,7 @@ class Player(GObject.GObject):
     def _on_eos(self, klass):
         if self._next_song:
             GLib.idle_add(self._on_glib_idle)
-        elif (self.repeat == RepeatType.NONE):
+        elif (self.repeat == RepeatMode.NONE):
             self.stop()
 
             if self.playlist is not None:
@@ -526,9 +505,9 @@ class Player(GObject.GObject):
     def play_pause(self):
         """Toggle play/pause state"""
         if self._player.state == Playback.PLAYING:
-            self.set_playing(False)
+            self.pause()
         else:
-            self.set_playing(True)
+            self.play()
 
     @log
     def _create_model(self, model, model_iter):
@@ -562,8 +541,15 @@ class Player(GObject.GObject):
         GLib.idle_add(self._validate_next_song)
 
     @log
-    def running_playlist(self, type, id):
-        if type == self.playlist_type and id == self.playlist_id:
+    def playing_playlist(self, type_, id_):
+        """Test if the current playlist matches type_ and id_.
+
+        :param string type_: playlist type_
+        :param string id_: unique identifer to recognize the playlist
+        :returns: True if these are the same playlists. False otherwise.
+        :rtype: bool
+        """
+        if type_ == self.playlist_type and id_ == self.playlist_id:
             return self.playlist
         else:
             return None
@@ -615,7 +601,6 @@ class Player(GObject.GObject):
         return self._player.state
 
     @GObject.Property
-    @log
     def url(self):
         """GstPlayer url loaded
 
