@@ -89,8 +89,15 @@ class Query():
 
     @staticmethod
     def all_playlists():
+        """Query to retrieve all playlists (user and smart ones)."""
         return Query.playlists('?playlist a nmm:Playlist .')
 
+    @staticmethod
+    def all_user_playlists():
+        """Query to retrieve playlists created by user."""
+        return Query.playlists(
+            '?playlist a nmm:Playlist .', 'FILTER (!BOUND(?tag))')
+
     @staticmethod
     def all_songs_count():
         query = """
@@ -108,20 +115,6 @@ class Query():
 
         return query
 
-    @staticmethod
-    def all_non_static_playlists_count():
-        query = """
-    SELECT
-        COUNT (?pl)
-    {   ?pl a nmm:Playlist .
-        OPTIONAL { ?pl nao:hasTag ?tag } .
-        FILTER (!BOUND(nfo:belongsToContainer(?pl)))
-        FILTER (!BOUND(?tag))
-    }
-        """.replace('\n', ' ').strip()
-
-        return query
-
     @staticmethod
     def albums(where_clause):
         query = """
@@ -222,7 +215,7 @@ class Query():
         return query
 
     @staticmethod
-    def playlists(where_clause):
+    def playlists(where_clause, filter_clause=None):
         query = """
     SELECT DISTINCT
         rdf:type(?playlist)
@@ -239,11 +232,13 @@ class Query():
                      && !STRENDS(LCASE(?url), '.m3u8')
                      && !STRENDS(LCASE(?url), '.pls')
                      || !BOUND(nfo:belongsToContainer(?playlist)) )
+            %(filter_clause)s
             OPTIONAL { ?playlist nao:hasTag ?tag }
         }
     ORDER BY DESC(tracker:added(?playlist)) !BOUND(?tag) LCASE(?title)
     """.replace('\n', ' ').strip() % {
             'where_clause': where_clause.replace('\n', ' ').strip(),
+            'filter_clause': filter_clause or '',
             'music_dir': Query.MUSIC_URI
         }
 
@@ -828,13 +823,16 @@ class Query():
                     BIND(tracker:normalize(nie:title(nmm:musicAlbum(?song)), 'nfkd') AS ?match1) .
                     BIND(tracker:normalize(nmm:artistName(nmm:performer(?song)), 'nfkd') AS ?match2) .
                     BIND(tracker:normalize(nie:title(?song), 'nfkd') AS ?match3) .
+                    BIND(tracker:normalize(nmm:composer(?song), 'nfkd') AS ?match4) .
                     FILTER (
                         CONTAINS(tracker:case-fold(tracker:unaccent(?match1)), "%(name)s") ||
                         CONTAINS(tracker:case-fold(?match1), "%(name)s") ||
                         CONTAINS(tracker:case-fold(tracker:unaccent(?match2)), "%(name)s") ||
                         CONTAINS(tracker:case-fold(?match2), "%(name)s") ||
                         CONTAINS(tracker:case-fold(tracker:unaccent(?match3)), "%(name)s") ||
-                        CONTAINS(tracker:case-fold(?match3), "%(name)s")
+                        CONTAINS(tracker:case-fold(?match3), "%(name)s") ||
+                        CONTAINS(tracker:case-fold(tracker:unaccent(?match4)), "%(name)s") ||
+                        CONTAINS(tracker:case-fold(?match4), "%(name)s")
                     )
                 }
             }
@@ -887,13 +885,16 @@ class Query():
                     BIND(tracker:normalize(nie:title(nmm:musicAlbum(?song)), 'nfkd') AS ?match1) .
                     BIND(tracker:normalize(nmm:artistName(nmm:performer(?song)), 'nfkd') AS ?match2) .
                     BIND(tracker:normalize(nie:title(?song), 'nfkd') AS ?match3) .
+                    BIND(tracker:normalize(nmm:composer(?song), 'nfkd') AS ?match4) .
                     FILTER (
                         CONTAINS(tracker:case-fold(tracker:unaccent(?match1)), "%(name)s") ||
                         CONTAINS(tracker:case-fold(?match1), "%(name)s") ||
                         CONTAINS(tracker:case-fold(tracker:unaccent(?match2)), "%(name)s") ||
                         CONTAINS(tracker:case-fold(?match2), "%(name)s") ||
                         CONTAINS(tracker:case-fold(tracker:unaccent(?match3)), "%(name)s") ||
-                        CONTAINS(tracker:case-fold(?match3), "%(name)s")
+                        CONTAINS(tracker:case-fold(?match3), "%(name)s") ||
+                        CONTAINS(tracker:case-fold(tracker:unaccent(?match4)), "%(name)s") ||
+                        CONTAINS(tracker:case-fold(?match4), "%(name)s")
                     )
                 }
             }
@@ -946,13 +947,16 @@ class Query():
                     BIND(tracker:normalize(nie:title(nmm:musicAlbum(?song)), 'nfkd') AS ?match1) .
                     BIND(tracker:normalize(nmm:artistName(nmm:performer(?song)), 'nfkd') AS ?match2) .
                     BIND(tracker:normalize(nie:title(?song), 'nfkd') AS ?match3) .
+                    BIND(tracker:normalize(nmm:composer(?song), 'nfkd') AS ?match4) .
                     FILTER (
                         CONTAINS(tracker:case-fold(tracker:unaccent(?match1)), "%(name)s") ||
                         CONTAINS(tracker:case-fold(?match1), "%(name)s") ||
                         CONTAINS(tracker:case-fold(tracker:unaccent(?match2)), "%(name)s") ||
                         CONTAINS(tracker:case-fold(?match2), "%(name)s") ||
                         CONTAINS(tracker:case-fold(tracker:unaccent(?match3)), "%(name)s") ||
-                        CONTAINS(tracker:case-fold(?match3), "%(name)s")
+                        CONTAINS(tracker:case-fold(?match3), "%(name)s") ||
+                        CONTAINS(tracker:case-fold(tracker:unaccent(?match4)), "%(name)s") ||
+                        CONTAINS(tracker:case-fold(?match4), "%(name)s")
                     )
                 }
             }
