@@ -24,7 +24,7 @@
 
 from gettext import gettext as _
 
-from gi.repository import Gdk, Gio, GLib, GObject, Gtk, Pango
+from gi.repository import Gio, GLib, GObject, Gtk, Pango
 
 from gnomemusic import log
 from gnomemusic.grilo import grilo
@@ -35,6 +35,7 @@ from gnomemusic.widgets.notificationspopup import PlaylistNotification
 from gnomemusic.widgets.playlistcontextmenu import PlaylistContextMenu
 from gnomemusic.widgets.playlistcontrols import PlaylistControls
 from gnomemusic.widgets.playlistdialog import PlaylistDialog
+from gnomemusic.widgets.sidebarrow import SidebarRow
 import gnomemusic.utils as utils
 
 playlists = Playlists.get_default()
@@ -291,13 +292,11 @@ class PlaylistView(BaseView):
             index = 0
 
         title = utils.get_media_title(playlist)
-        row = Gtk.ListBoxRow()
+        row = SidebarRow()
+        row.props.text = title
+        # FIXME: Passing the Grl.Media with the row object is ugly.
         row.playlist = playlist
-        label = Gtk.Label(
-            label=title, xalign=0, xpad=16, ypad=16,
-            ellipsize=Pango.EllipsizeMode.END)
-        row.add(label)
-        row.show_all()
+
         self._sidebar.insert(row, index)
         self._offset += 1
 
@@ -343,15 +342,16 @@ class PlaylistView(BaseView):
         :param Gtk.TreeView treeview: self._view
         :param Gdk.EventButton event: clicked event
         """
-        if event.button != Gdk.BUTTON_SECONDARY:
+        if not event.triggers_context_menu():
             return
 
-        path, col, cell_x, cell_y = treeview.get_path_at_pos(event.x, event.y)
+        (coord_x, coord_y) = event.get_coords()
+        path, col, cell_x, cell_y = treeview.get_path_at_pos(coord_x, coord_y)
         self._view.get_selection().select_path(path)
 
         rect = self._view.get_visible_rect()
-        rect.x = event.x - rect.width / 2.0
-        rect.y = event.y - rect.height + 5
+        rect.x = coord_x - rect.width / 2.0
+        rect.y = coord_y - rect.height + 5
 
         self._song_popover.set_relative_to(self._view)
         self._song_popover.set_pointing_to(rect)
