@@ -214,14 +214,14 @@ class SongsView(BaseView):
         :param Gdk.EventButton event: clicked event
         """
         modifiers = Gtk.accelerator_get_default_mod_mask()
-        if ((event.state & modifiers) == Gdk.ModifierType.CONTROL_MASK
+        if ((event.get_state() & modifiers) == Gdk.ModifierType.CONTROL_MASK
                 and not self.props.selection_mode):
             self._on_selection_mode_request()
 
         if (self.selection_mode
                 and not self._star_handler.star_renderer_click):
             path, col, cell_x, cell_y = treeview.get_path_at_pos(
-                event.x, event.y)
+                *event.get_coords())
             iter_ = self.model.get_iter(path)
             self.model[iter_][6] = not self.model[iter_][6]
             self._update_header_from_selection(len(self.get_selected_songs()))
