@@ -29,7 +29,7 @@ from gnomemusic import log
 from gnomemusic.albumartcache import Art
 from gnomemusic.gstplayer import Playback
 from gnomemusic.player import RepeatMode
-from gnomemusic.widgets.coverstack import CoverStack
+from gnomemusic.widgets.coverstack import CoverStack  # noqa: F401
 from gnomemusic.widgets.smoothscale import SmoothScale  # noqa: F401
 import gnomemusic.utils as utils
 
@@ -71,7 +71,7 @@ class PlayerToolbar(Gtk.ActionBar):
         self._player = player
         self._progress_scale.player = self._player.get_gst_player()
 
-        self._cover_stack = CoverStack(self._cover_stack, Art.Size.XSMALL)
+        self._cover_stack.props.size = Art.Size.XSMALL
         self._cover_stack.connect('updated', self._on_cover_stack_updated)
 
         self._sync_repeat_image()
