@@ -33,6 +33,7 @@ from gi.repository import Gtk, Gdk, Gio, GLib
 from gettext import gettext as _
 
 from gnomemusic import log
+from gnomemusic.mediakeys import MediaKeys
 from gnomemusic.player import Player, RepeatMode
 from gnomemusic.query import Query
 from gnomemusic.utils import View
@@ -96,11 +97,10 @@ class Window(Gtk.ApplicationWindow):
         self.notifications_popup = NotificationsPopup()
         self._overlay.add_overlay(self.notifications_popup)
 
-        self._media_keys_proxy = None
-        self._init_media_keys_proxy()
+        MediaKeys(self.player, self)
 
         self.window_size_update_timeout = None
-        self.connect("window-state-event", self._on_window_state_event)
+        self.connect("notify::is-maximized", self._on_maximized)
         self.connect("configure-event", self._on_configure_event)
         grilo.connect('changes-pending', self._on_changes_pending)
 
@@ -143,66 +143,8 @@ class Window(Gtk.ApplicationWindow):
         return False
 
     @log
-    def _on_window_state_event(self, widget, event):
-        self.settings.set_boolean('window-maximized', 'GDK_WINDOW_STATE_MAXIMIZED' in event.new_window_state.value_names)
-
-    @log
-    def _init_media_keys_proxy(self):
-        def name_appeared(connection, name, name_owner, data=None):
-            Gio.DBusProxy.new_for_bus(
-                Gio.BusType.SESSION,
-                Gio.DBusProxyFlags.DO_NOT_LOAD_PROPERTIES, None,
-                "org.gnome.SettingsDaemon.MediaKeys",
-                "/org/gnome/SettingsDaemon/MediaKeys",
-                "org.gnome.SettingsDaemon.MediaKeys", None,
-                self._media_keys_proxy_ready)
-
-        Gio.bus_watch_name(
-            Gio.BusType.SESSION, "org.gnome.SettingsDaemon.MediaKeys",
-            Gio.BusNameWatcherFlags.NONE, name_appeared, None)
-
-    @log
-    def _media_keys_proxy_ready(self, proxy, result, data=None):
-        try:
-            self._media_keys_proxy = proxy.new_finish(result)
-        except GLib.Error as e:
-            logger.warning(
-                "Error: Failed to contact settings daemon:", e.message)
-            return
-
-        self._grab_media_player_keys()
-        self._media_keys_proxy.connect(
-            "g-signal", self._handle_media_keys)
-        self.connect("focus-in-event", self._grab_media_player_keys)
-
-    @log
-    def _grab_media_player_keys(self, window=None, event=None):
-        def proxy_call_finished(proxy, result, data=None):
-            try:
-                proxy.call_finish(result)
-            except GLib.Error as e:
-                logger.warning(
-                    "Error: Failed to grab mediaplayer keys: {}".format(
-                        e.message))
-
-        self._media_keys_proxy.call(
-            "GrabMediaPlayerKeys", GLib.Variant("(su)", ("Music", 0)),
-            Gio.DBusCallFlags.NONE, -1, None, proxy_call_finished)
-
-    @log
-    def _handle_media_keys(self, proxy, sender, signal, parameters):
-        app, response = parameters.unpack()
-        if app != "Music":
-            return
-
-        if "Play" in response:
-            self.player.play_pause()
-        elif "Stop" in response:
-            self.player.stop()
-        elif "Next" in response:
-            self.player.next()
-        elif "Previous" in response:
-            self.player.previous()
+    def _on_maximized(self, klass, value, data=None):
+        self.settings.set_boolean('window-maximized', self.is_maximized())
 
     @log
     def _setup_view(self):
@@ -230,8 +172,10 @@ class Window(Gtk.ApplicationWindow):
         # bottom line of the searchbar
         self._stack.get_style_context().add_class('background')
 
-        self._overlay = Gtk.Overlay(child=self._stack)
-        self._overlay.add_overlay(self.headerbar.dropdown)
+        self._overlay = Gtk.Overlay()
+        self._overlay.add(self._stack)
+        # FIXME: Need to find a proper way to do this.
+        self._overlay.add_overlay(self.headerbar.searchbar._dropdown)
         self.set_titlebar(self.headerbar)
         self._box.pack_start(self.headerbar.searchbar, False, False, 0)
         self._box.pack_start(self._overlay, True, True, 0)
@@ -307,7 +251,6 @@ class Window(Gtk.ApplicationWindow):
         self.headerbar.props.state = HeaderBar.State.MAIN
         self.headerbar.props.stack = self._stack
         self.headerbar.searchbar.show()
-        self.headerbar.dropdown.show()
 
     @log
     def _select_all(self, action=None, param=None):
@@ -333,98 +276,94 @@ class Window(Gtk.ApplicationWindow):
 
     @log
     def _on_key_press(self, widget, event):
-        modifiers = Gtk.accelerator_get_default_mod_mask()
-        modifiers = (event.state & modifiers)
+        modifiers = event.get_state()
+        (_, keyval) = event.get_keyval()
 
-        if modifiers != 0:
-            control_mask = Gdk.ModifierType.CONTROL_MASK
-            shift_mask = Gdk.ModifierType.SHIFT_MASK
-            mod1_mask = Gdk.ModifierType.MOD1_MASK
+        control_mask = Gdk.ModifierType.CONTROL_MASK
+        shift_mask = Gdk.ModifierType.SHIFT_MASK
+        mod1_mask = Gdk.ModifierType.MOD1_MASK
+        shift_ctrl_mask = control_mask | shift_mask
 
-            if (event.keyval == Gdk.KEY_a
-                    and modifiers == control_mask):
+        # Ctrl+<KEY>
+        if control_mask == modifiers:
+            if keyval == Gdk.KEY_a:
                 self._select_all()
-            if (event.keyval == Gdk.KEY_A
-                    and modifiers == (shift_mask | control_mask)):
-                self._select_none()
             # Open search bar on Ctrl + F
-            if ((event.keyval == Gdk.KEY_f and modifiers == control_mask)
+            if (keyval == Gdk.KEY_f
                     and not self.views[View.PLAYLIST].rename_active
                     and self.headerbar.props.state != HeaderBar.State.SEARCH):
                 self.headerbar.searchbar.toggle()
             # Play / Pause on Ctrl + SPACE
-            if (event.keyval == Gdk.KEY_space
-                    and modifiers == control_mask):
+            if keyval == Gdk.KEY_space:
                 self.player.play_pause()
             # Play previous on Ctrl + B
-            if (event.keyval == Gdk.KEY_b
-                    and modifiers == control_mask):
+            if keyval == Gdk.KEY_b:
                 self.player.previous()
             # Play next on Ctrl + N
-            if (event.keyval == Gdk.KEY_n
-                    and modifiers == control_mask):
+            if keyval == Gdk.KEY_n:
                 self.player.next()
             # Toggle repeat on Ctrl + R
-            if (event.keyval == Gdk.KEY_r
-                    and modifiers == control_mask):
+            if keyval == Gdk.KEY_r:
                 if self.player.get_repeat_mode() == RepeatMode.SONG:
                     self.player.set_repeat_mode(RepeatMode.NONE)
                 else:
                     self.player.set_repeat_mode(RepeatMode.SONG)
             # Toggle shuffle on Ctrl + S
-            if (event.keyval == Gdk.KEY_s
-                    and modifiers == control_mask):
+            if keyval == Gdk.KEY_s:
                 if self.player.get_repeat_mode() == RepeatMode.SHUFFLE:
                     self.player.set_repeat_mode(RepeatMode.NONE)
                 else:
                     self.player.set_repeat_mode(RepeatMode.SHUFFLE)
-            # Go back from Album view on Alt + Left
-            if (event.keyval == Gdk.KEY_Left
-                    and modifiers == mod1_mask):
-                self.headerbar._on_back_button_clicked()
-            if ((event.keyval in [Gdk.KEY_1, Gdk.KEY_KP_1])
-                    and modifiers == control_mask):
+            # Headerbar switching
+            if keyval in [Gdk.KEY_1, Gdk.KEY_KP_1]:
                 self._toggle_view(View.ALBUM)
-            if ((event.keyval in [Gdk.KEY_2, Gdk.KEY_KP_2])
-                    and modifiers == control_mask):
+            if keyval in [Gdk.KEY_2, Gdk.KEY_KP_2]:
                 self._toggle_view(View.ARTIST)
-            if ((event.keyval in [Gdk.KEY_3, Gdk.KEY_KP_3])
-                    and modifiers == control_mask):
+            if keyval in [Gdk.KEY_3, Gdk.KEY_KP_3]:
                 self._toggle_view(View.SONG)
-            if ((event.keyval in [Gdk.KEY_4, Gdk.KEY_KP_4])
-                    and modifiers == control_mask):
+            if keyval in [Gdk.KEY_4, Gdk.KEY_KP_4]:
                 self._toggle_view(View.PLAYLIST)
+        # Ctrl+Shift+<KEY>
+        elif modifiers == shift_ctrl_mask:
+            if keyval == Gdk.KEY_A:
+                self._select_none()
+        # Alt+<KEY>
+        elif modifiers == mod1_mask:
+            # Go back from Album view on Alt + Left
+            if keyval == Gdk.KEY_Left:
+                self.headerbar._on_back_button_clicked()
+        # No modifier
         else:
-            if (event.keyval == Gdk.KEY_AudioPlay
-                    or event.keyval == Gdk.KEY_AudioPause):
+            if (keyval == Gdk.KEY_AudioPlay
+                    or keyval == Gdk.KEY_AudioPause):
                 self.player.play_pause()
 
-            if event.keyval == Gdk.KEY_AudioStop:
+            if keyval == Gdk.KEY_AudioStop:
                 self.player.stop()
 
-            if event.keyval == Gdk.KEY_AudioPrev:
+            if keyval == Gdk.KEY_AudioPrev:
                 self.player.previous()
 
-            if event.keyval == Gdk.KEY_AudioNext:
+            if keyval == Gdk.KEY_AudioNext:
                 self.player.next()
 
             child = self._stack.get_visible_child()
-            if (event.keyval == Gdk.KEY_Delete
+            if (keyval == Gdk.KEY_Delete
                     and child == self.views[View.PLAYLIST]):
                 self.views[View.PLAYLIST].remove_playlist()
             # Close selection mode or search bar after Esc is pressed
-            if event.keyval == Gdk.KEY_Escape:
+            if keyval == Gdk.KEY_Escape:
                 if self.headerbar.props.selection_mode:
                     self.headerbar.props.selection_mode = False
                 else:
                     self.headerbar.searchbar.reveal(False)
 
         # Open the search bar when typing printable chars.
-        key_unic = Gdk.keyval_to_unicode(event.keyval)
+        key_unic = Gdk.keyval_to_unicode(keyval)
         if ((not self.headerbar.searchbar.get_search_mode()
-                and not event.keyval == Gdk.KEY_space)
+                and not keyval == Gdk.KEY_space)
                 and GLib.unichar_isprint(chr(key_unic))
-                and (modifiers == Gdk.ModifierType.SHIFT_MASK
+                and (modifiers == shift_mask
                      or modifiers == 0)
                 and not self.views[View.PLAYLIST].rename_active
                 and self.headerbar.props.state != HeaderBar.State.SEARCH):
